/****************************************************************************/
/*                                                                          */
/*                   Copyright 2003 University of Glasgow                   */
/*                                                                          */
/*                           All rights reserved.                           */
/*                                                                          */
/****************************************************************************/

/****************************************************************************/
/* FILE          : Main.java                                                */
/* DESCRIPTION   : Top-level driver for the Better Functional Language      */
/*                                                                          */
/* AUTHOR        : S.J.Gay                                                  */
/* DATE          : 28th August 2003                                         */
/*                                                                          */
/* LAST MODIFIED : S.J.Gay                                                  */
/* DATE          : 27th October 2003                                        */
/****************************************************************************/

package bfl;

import bfl.node.*;
import bfl.lexer.*;
import bfl.parser.*;
import bfl.error.*;
import bfl.checker.*;
import bfl.types.*;
import java.io.*;
import java.util.*;

public class Main {

    private static Lexer scanner;
    private static Parser parser;

    /** The AST representing the source program. */
    private static Node theAST;

    static void check (String sourceName) {
	try {
	    scanner = new Lexer(new PushbackReader(
		      new BufferedReader(new FileReader(sourceName)), 1024));
	} catch(FileNotFoundException e)
	    {System.out.println("Can't access source file " + sourceName);
	    System.exit(1);};

	parser = new Parser(scanner);
        try {
	    theAST = parser.parse();
	} catch(Exception e)
	    {System.out.println(e);
	     System.exit(1);}

	Checker chk = new Checker(); 
	theAST.apply(chk);
	ErrorTable errorTable = chk.getErrorTable();
	if (errorTable.hasErrors()) {
	    errorTable.output(System.out,
			      "while processing " + sourceName);
	    System.exit(1);
	}
	else {
	    System.out.println("Expression has type " +
			       ((Type)chk.getOut(theAST)).name());
	}
    }

    public static void main(String[] args) {
        if (args.length < 1) {
            System.out.println("Usage: check filename");
            System.exit(1);
        }
        check(args[0]);
    }
}
