/****************************************************************************/
/*                                                                          */
/*                   Copyright 2003 University of Glasgow                   */
/*                                                                          */
/*                           All rights reserved.                           */
/*                                                                          */
/****************************************************************************/

/****************************************************************************/
/* FILE          : Env.java                                                 */
/* DESCRIPTION   : Environment of identifier bindings                       */
/*                                                                          */
/* AUTHOR        : S.J.Gay                                                  */
/* DATE          : 29th August 2003                                         */
/*                                                                          */
/* LAST MODIFIED : S.J.Gay                                                  */
/* DATE          : 27th October 2003                                        */
/****************************************************************************/

package bfl.env;

import bfl.node.*;
import bfl.types.*;
import java.util.*;

/* An environment is a vector of hash tables, used as a stack.
The 'put' method adds an entry to the hash table at the top of the stack
(first element of the vector). The 'get' method searches down the stack.
The 'openScope' method creates a new hash table on the top of the stack.
The 'closeScope' method removes the top-most hash table. */

public class Env {

    Vector stack;

    public Env() {
	stack = new Vector(1,1);
	stack.addElement(new Hashtable());
    }

    public void openScope() {
	stack.addElement(new Hashtable());
    }

    public void closeScope() {
	stack.removeElementAt(stack.size()-1);
    }

    public EnvEntry put(String n, EnvEntry e) {
	return (EnvEntry)(((Hashtable)stack.elementAt(stack.size()-1))
			  .put(n,e));
    }

    public EnvEntry get(String n) {
	for (int i = stack.size()-1; i >= 0; i--) {
	    EnvEntry e = (EnvEntry)((Hashtable)stack.elementAt(i)).get(n);
	    if (e != null)
		return e;
	};
	return null;
    }

}
